@@ -93,7 +93,7 @@ def extract_wav_headers(data):
     # def search_subchunk(data, subchunk_id):
     pos = 12  # The size of the RIFF chunk descriptor
     subchunks = []
-    while pos + 8 < len(data) and len(subchunks) < 10:
+    while pos + 8 <= len(data) and len(subchunks) < 10:
         subchunk_id = data[pos:pos + 4]
         subchunk_size = struct.unpack_from('<I', data[pos + 4:pos + 8])[0]
         subchunks.append(WavSubChunk(subchunk_id, pos, subchunk_size))
@@ -137,6 +137,10 @@ def fix_wav_headers(data):
     if not headers or headers[-1].id != b'data':
         return
 
+    # TODO: Handle huge files in some other way
+    if len(data) > 2**32:
+        raise CouldntDecodeError("Unable to process >4GB files")
+
     # Set the file size in the RIFF chunk descriptor
     data[4:8] = struct.pack('<I', len(data) - 8)
 
@@ -224,6 +228,9 @@ class AudioSegment(object):
             self.frame_rate = wav_data.sample_rate
             self.frame_width = self.channels * self.sample_width
             self._data = wav_data.raw_data
+            if self.sample_width == 1:
+                # convert from unsigned integers in wav
+                self._data = audioop.bias(self._data, 1, -128)
 
         # Convert 24-bit audio to 32-bit audio.
         # (stdlib audioop and array modules do not support 24-bit data)
@@ -282,7 +289,7 @@ class AudioSegment(object):
             return False
 
     def __hash__(self):
-       return hash(AudioSegment) ^ hash((self.channels, self.frame_rate, self.sample_width, self._data))
+        return hash(AudioSegment) ^ hash((self.channels, self.frame_rate, self.sample_width, self._data))
 
     def __ne__(self, other):
         return not (self == other)
@@ -498,7 +505,7 @@ class AudioSegment(object):
     @classmethod
     def from_file_using_temporary_files(cls, file, format=None, codec=None, parameters=None, **kwargs):
         orig_file = file
-        file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
+        file, close_file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
 
         if format:
             format = format.lower()
@@ -517,7 +524,8 @@ class AudioSegment(object):
         if is_format("wav"):
             try:
                 obj = cls._from_safe_wav(file)
-                file.close()
+                if close_file:
+                    file.close()
                 return obj
             except:
                 file.seek(0)
@@ -532,7 +540,8 @@ class AudioSegment(object):
                 'frame_width': channels * sample_width
             }
             obj = cls(data=file.read(), metadata=metadata)
-            file.close()
+            if close_file:
+                file.close()
             return obj
 
         input_file = NamedTemporaryFile(mode='wb', delete=False)
@@ -542,14 +551,17 @@ class AudioSegment(object):
             input_file.flush()
             input_file.close()
             input_file = NamedTemporaryFile(mode='wb', delete=False, buffering=2 ** 31 - 1)
-            file.close()
+            if close_file:
+                file.close()
+            close_file = True
             file = open(orig_file, buffering=2 ** 13 - 1, mode='rb')
             reader = file.read(2 ** 31 - 1)
             while reader:
                 input_file.write(reader)
                 reader = file.read(2 ** 31 - 1)
         input_file.flush()
-        file.close()
+        if close_file:
+            file.close()
 
         output = NamedTemporaryFile(mode="rb", delete=False)
 
@@ -590,7 +602,7 @@ class AudioSegment(object):
             if p.returncode != 0:
                 raise CouldntDecodeError(
                     "Decoding failed. ffmpeg returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(
-                        p.returncode, p_err))
+                        p.returncode, p_err.decode(errors='ignore') ))
             obj = cls._from_safe_wav(output)
         finally:
             input_file.close()
@@ -607,7 +619,7 @@ class AudioSegment(object):
             filename = fsdecode(file)
         except TypeError:
             filename = None
-        file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
+        file, close_file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
 
         if format:
             format = format.lower()
@@ -653,29 +665,37 @@ class AudioSegment(object):
             # force audio decoder
             conversion_command += ["-acodec", codec]
 
+        read_ahead_limit = kwargs.get('read_ahead_limit', -1)
         if filename:
             conversion_command += ["-i", filename]
             stdin_parameter = None
             stdin_data = None
         else:
-            conversion_command += ["-i", "-"]
+            if cls.converter == 'ffmpeg':
+                conversion_command += ["-read_ahead_limit", str(read_ahead_limit),
+                                       "-i", "cache:pipe:0"]
+            else:
+                conversion_command += ["-i", "-"]
             stdin_parameter = subprocess.PIPE
             stdin_data = file.read()
 
-        info = mediainfo_json(orig_file)
+        if codec:
+            info = None
+        else:
+            info = mediainfo_json(orig_file, read_ahead_limit=read_ahead_limit)
         if info:
             audio_streams = [x for x in info['streams']
                              if x['codec_type'] == 'audio']
             # This is a workaround for some ffprobe versions that always say
             # that mp3/mp4/aac/webm/ogg files contain fltp samples
+            audio_codec = audio_streams[0].get('codec_name')
             if (audio_streams[0].get('sample_fmt') == 'fltp' and
-                    (is_format("mp3") or is_format("mp4") or is_format("aac") or
-                     is_format("webm") or is_format("ogg"))):
+                    audio_codec in ['mp3', 'mp4', 'aac', 'webm', 'ogg']):
                 bits_per_sample = 16
             else:
                 bits_per_sample = audio_streams[0]['bits_per_sample']
             if bits_per_sample == 8:
-                acodec = 'pcm_s8'
+                acodec = 'pcm_u8'
             else:
                 acodec = 'pcm_s%dle' % bits_per_sample
 
@@ -698,16 +718,18 @@ class AudioSegment(object):
         p_out, p_err = p.communicate(input=stdin_data)
 
         if p.returncode != 0 or len(p_out) == 0:
-            file.close()
+            if close_file:
+                file.close()
             raise CouldntDecodeError(
                 "Decoding failed. ffmpeg returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(
-                    p.returncode, p_err))
+                    p.returncode, p_err.decode(errors='ignore') ))
 
         p_out = bytearray(p_out)
         fix_wav_headers(p_out)
         obj = cls._from_safe_wav(BytesIO(p_out))
 
-        file.close()
+        if close_file:
+            file.close()
 
         return obj
 
@@ -734,10 +756,11 @@ class AudioSegment(object):
 
     @classmethod
     def _from_safe_wav(cls, file):
-        file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
+        file, close_file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
         file.seek(0)
         obj = cls(data=file)
-        file.close()
+        if close_file:
+            file.close()
         return obj
 
     def export(self, out_f=None, format='mp3', codec=None, bitrate=None, parameters=None, tags=None, id3v2_version='4',
@@ -754,7 +777,7 @@ class AudioSegment(object):
             ('mp3', 'wav', 'raw', 'ogg' or other ffmpeg/avconv supported files)
 
         codec (string)
-            Codec used to encoding for the destination.
+            Codec used to encode the destination file.
 
         bitrate (string)
             Bitrate used when encoding destination file. (64, 92, 128, 256, 312k...)
@@ -762,7 +785,7 @@ class AudioSegment(object):
             ffmpeg documentation for details (bitrate usually shown as -b, -ba or
             -a:b).
 
-        parameters (string)
+        parameters (list of strings)
             Aditional ffmpeg/avconv parameters
 
         tags (dict)
@@ -777,7 +800,13 @@ class AudioSegment(object):
         """
         id3v2_allowed_versions = ['3', '4']
 
-        out_f = _fd_or_path_or_tempfile(out_f, 'wb+')
+        if format == "raw" and (codec is not None or parameters is not None):
+            raise AttributeError(
+                    'Can not invoke ffmpeg when export format is "raw"; '
+                    'specify an ffmpeg raw format like format="s16le" instead '
+                    'or call export(format="raw") with no codec or parameters')
+
+        out_f, _ = _fd_or_path_or_tempfile(out_f, 'wb+')
         out_f.seek(0)
 
         if format == "raw":
@@ -785,12 +814,19 @@ class AudioSegment(object):
             out_f.seek(0)
             return out_f
 
-        # for wav output we can just write the data directly to out_f
-        if format == "wav":
+        # wav with no ffmpeg parameters can just be written directly to out_f
+        easy_wav = format == "wav" and codec is None and parameters is None
+
+        if easy_wav:
             data = out_f
         else:
             data = NamedTemporaryFile(mode="wb", delete=False)
 
+        pcm_for_wav = self._data
+        if self.sample_width == 1:
+            # convert to unsigned integers for wav
+            pcm_for_wav = audioop.bias(self._data, 1, 128)
+
         wave_data = wave.open(data, 'wb')
         wave_data.setnchannels(self.channels)
         wave_data.setsampwidth(self.sample_width)
@@ -798,11 +834,11 @@ class AudioSegment(object):
         # For some reason packing the wave header struct with
         # a float in python 2 doesn't throw an exception
         wave_data.setnframes(int(self.frame_count()))
-        wave_data.writeframesraw(self._data)
+        wave_data.writeframesraw(pcm_for_wav)
         wave_data.close()
 
-        # for wav files, we're done (wav data is written directly to out_f)
-        if format == 'wav':
+        # for easy wav files, we're done (wav data is written directly to out_f)
+        if easy_wav:
             return out_f
 
         output = NamedTemporaryFile(mode="w+b", delete=False)
@@ -874,7 +910,7 @@ class AudioSegment(object):
         if p.returncode != 0:
             raise CouldntEncodeError(
                 "Encoding failed. ffmpeg/avlib returned error code: {0}\n\nCommand:{1}\n\nOutput from ffmpeg/avlib:\n\n{2}".format(
-                    p.returncode, conversion_command, p_err))
+                    p.returncode, conversion_command, p_err.decode(errors='ignore') ))
 
         output.seek(0)
         out_f.write(output.read())
@@ -907,20 +943,12 @@ class AudioSegment(object):
         if sample_width == self.sample_width:
             return self
 
-        data = self._data
-
-        if self.sample_width == 1:
-            data = audioop.bias(data, 1, -128)
-
-        if data:
-            data = audioop.lin2lin(data, self.sample_width, sample_width)
-
-        if sample_width == 1:
-            data = audioop.bias(data, 1, 128)
-
         frame_width = self.channels * sample_width
-        return self._spawn(data, overrides={'sample_width': sample_width,
-                                            'frame_width': frame_width})
+
+        return self._spawn(
+            audioop.lin2lin(self._data, self.sample_width, sample_width),
+            overrides={'sample_width': sample_width, 'frame_width': frame_width}
+        )
 
     def set_frame_rate(self, frame_rate):
         if frame_rate == self.frame_rate:
@@ -944,12 +972,29 @@ class AudioSegment(object):
             fn = audioop.tostereo
             frame_width = self.frame_width * 2
             fac = 1
+            converted = fn(self._data, self.sample_width, fac, fac)
         elif channels == 1 and self.channels == 2:
             fn = audioop.tomono
             frame_width = self.frame_width // 2
             fac = 0.5
-
-        converted = fn(self._data, self.sample_width, fac, fac)
+            converted = fn(self._data, self.sample_width, fac, fac)
+        elif channels == 1:
+            channels_data = [seg.get_array_of_samples() for seg in self.split_to_mono()]
+            frame_count = int(self.frame_count())
+            converted = array.array(
+                channels_data[0].typecode,
+                b'\0' * (frame_count * self.sample_width)
+            )
+            for raw_channel_data in channels_data:
+                for i in range(frame_count):
+                    converted[i] += raw_channel_data[i] // self.channels
+            frame_width = self.frame_width // self.channels
+        elif self.channels == 1:
+            dup_channels = [self for iChannel in range(channels)]
+            return AudioSegment.from_mono_audiosegments(*dup_channels)
+        else:
+            raise ValueError(
+                "AudioSegment.set_channels only supports mono-to-multi channel and multi-to-mono channel conversion")
 
         return self._spawn(data=converted,
                            overrides={
@@ -979,10 +1024,7 @@ class AudioSegment(object):
 
     @property
     def rms(self):
-        if self.sample_width == 1:
-            return self.set_sample_width(2).rms
-        else:
-            return audioop.rms(self._data, self.sample_width)
+        return audioop.rms(self._data, self.sample_width)
 
     @property
     def dBFS(self):
