@@ -22,7 +22,7 @@ def detect_silence(audio_segment, min_silence_len=1000, silence_thresh=-16, seek
     slice_starts = range(0, last_slice_start + 1, seek_step)
 
     # guarantee last_slice_start is included in the range
-    # to make sure the last portion of the audio is seached
+    # to make sure the last portion of the audio is searched
     if last_slice_start % seek_step:
         slice_starts = itertools.chain(slice_starts, [last_slice_start])
 
@@ -99,18 +99,57 @@ def split_on_silence(audio_segment, min_silence_len=1000, silence_thresh=-16, ke
     silence_thresh - (in dBFS) anything quieter than this will be
         considered silence. default: -16dBFS
 
-    keep_silence - (in ms) amount of silence to leave at the beginning
-        and end of the chunks. Keeps the sound from sounding like it is
-        abruptly cut off. (default: 100ms)
+    keep_silence - (in ms or True/False) leave some silence at the beginning
+        and end of the chunks. Keeps the sound from sounding like it
+        is abruptly cut off.
+        When the length of the silence is less than the keep_silence duration
+        it is split evenly between the preceding and following non-silent
+        segments.
+        If True is specified, all the silence is kept, if False none is kept.
+        default: 100ms
     """
 
-    not_silence_ranges = detect_nonsilent(audio_segment, min_silence_len, silence_thresh, seek_step)
+    # from the itertools documentation
+    def pairwise(iterable):
+        "s -> (s0,s1), (s1,s2), (s2, s3), ..."
+        a, b = itertools.tee(iterable)
+        next(b, None)
+        return zip(a, b)
 
-    chunks = []
-    for start_i, end_i in not_silence_ranges:
-        start_i = max(0, start_i - keep_silence)
-        end_i += keep_silence
+    if isinstance(keep_silence, bool):
+        keep_silence = len(audio_segment) if keep_silence else 0
+
+    output_ranges = [
+        [ start - keep_silence, end + keep_silence ]
+        for (start,end)
+            in detect_nonsilent(audio_segment, min_silence_len, silence_thresh, seek_step)
+    ]
+
+    for range_i, range_ii in pairwise(output_ranges):
+        last_end = range_i[1]
+        next_start = range_ii[0]
+        if next_start < last_end:
+            range_i[1] = (last_end+next_start)//2
+            range_ii[0] = range_i[1]
+
+    return [
+        audio_segment[ max(start,0) : min(end,len(audio_segment)) ]
+        for start,end in output_ranges
+    ]
+
+
+def detect_leading_silence(sound, silence_threshold=-50.0, chunk_size=10):
+    '''
+    sound is a pydub.AudioSegment
+    silence_threshold in dB
+    chunk_size in ms
+    iterate over chunks until you find the first one with sound
+    '''
+    trim_ms = 0 # ms
+    assert chunk_size > 0 # to avoid infinite loop
+    while sound[trim_ms:trim_ms+chunk_size].dBFS < silence_threshold and trim_ms < len(sound):
+        trim_ms += chunk_size
+
+    return trim_ms
 
-        chunks.append(audio_segment[start_i:end_i])
 
-    return chunks
