@@ -18,6 +18,8 @@ from pydub.utils import (
     make_chunks,
     mediainfo,
     get_encoder_name,
+    get_supported_decoders,
+    get_supported_encoders,
 )
 from pydub.exceptions import (
     InvalidTag,
@@ -28,6 +30,7 @@ from pydub.exceptions import (
 )
 from pydub.silence import (
     detect_silence,
+    split_on_silence,
 )
 from pydub.generators import (
     Sine,
@@ -132,16 +135,16 @@ if sys.version_info >= (3, 6):
                 _ = AudioSegment.from_file(path)
 
         def assertWithinRange(self, val, lower_bound, upper_bound):
-            self.assertTrue(lower_bound < val < upper_bound,
+            self.assertTrue(lower_bound <= val <= upper_bound,
                             "%s is not in the acceptable range: %s - %s" %
                             (val, lower_bound, upper_bound))
 
         def assertWithinTolerance(self, val, expected, tolerance=None,
                                   percentage=None):
             if percentage is not None:
-                tolerance = val * percentage
-            lower_bound = val - tolerance
-            upper_bound = val + tolerance
+                tolerance = expected * percentage
+            lower_bound = expected - tolerance
+            upper_bound = expected + tolerance
             self.assertWithinRange(val, lower_bound, upper_bound)
 
         def test_export_pathlib_path(self):
@@ -208,16 +211,16 @@ class AudioSegmentTests(unittest.TestCase):
         self.png_cover_path = os.path.join(data_dir, 'cover.png')
 
     def assertWithinRange(self, val, lower_bound, upper_bound):
-        self.assertTrue(lower_bound < val < upper_bound,
+        self.assertTrue(lower_bound <= val <= upper_bound,
                         "%s is not in the acceptable range: %s - %s" %
                         (val, lower_bound, upper_bound))
 
     def assertWithinTolerance(self, val, expected, tolerance=None,
                               percentage=None):
         if percentage is not None:
-            tolerance = val * percentage
-        lower_bound = val - tolerance
-        upper_bound = val + tolerance
+            tolerance = expected * percentage
+        lower_bound = expected - tolerance
+        upper_bound = expected + tolerance
         self.assertWithinRange(val, lower_bound, upper_bound)
 
     def test_direct_instantiation_with_bytes(self):
@@ -241,6 +244,32 @@ class AudioSegmentTests(unittest.TestCase):
         # the data length should have grown by exactly 4:3 (24 bits turn into 32 bits)
         self.assertEqual(len(seg24.raw_data) * 3, len24 * 4)
 
+    def test_8_bit_audio(self):
+        original_path = os.path.join(data_dir,'test1.wav')
+        original_segment = AudioSegment.from_file(original_path)
+        target_rms = original_segment.rms//2**8
+
+        path_with_8bits = os.path.join(data_dir,'test1-8bit.wav')
+
+        def check_8bit_segment(segment):
+            self.assertWithinTolerance(segment.rms,target_rms,tolerance=0)
+
+        # check reading directly
+        check_8bit_segment(AudioSegment.from_file(path_with_8bits))
+
+        # check using ffmpeg on it
+        with open(path_with_8bits,'rb') as file_8bit:
+            check_8bit_segment(AudioSegment.from_file(file_8bit))
+
+        # check conversion from higher-width sample
+        check_8bit_segment(AudioSegment.from_file(original_path).set_sample_width(1))
+
+        # check audio export
+        with NamedTemporaryFile('w+b', suffix='.wav') as tmp_file:
+            original_segment.set_sample_width(1).export(tmp_file,format='wav')
+            tmp_file.seek(0)
+            check_8bit_segment(AudioSegment.from_file(tmp_file))
+
     def test_192khz_audio(self):
         test_files = [('test-192khz-16bit.wav', 16),
                       ('test-192khz-24bit.wav', 32),
@@ -456,11 +485,23 @@ class AudioSegmentTests(unittest.TestCase):
 
         self.assertEqual(len(mono), len(self.seg2))
 
-        monomp3 = AudioSegment.from_mp3(mono.export())
-        self.assertWithinTolerance(len(monomp3), len(self.seg2),
-                                   percentage=0.01)
+        with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_file:
+            if sys.platform == 'win32':
+                tmp_file.close()
 
-        merged = monomp3.append(stereo, crossfade=100)
+            mono.export(tmp_file.name, 'mp3')
+            monomp3 = AudioSegment.from_mp3(tmp_file.name)
+
+            self.assertWithinTolerance(
+                len(monomp3),
+                len(self.seg2),
+                tolerance=105
+            )
+
+            if sys.platform == 'win32':
+                os.remove(tmp_file.name)
+
+        merged = mono.append(stereo, crossfade=100)
         self.assertWithinTolerance(len(merged),
                                    len(self.seg1) + len(self.seg2) - 100,
                                    tolerance=1)
@@ -558,6 +599,27 @@ class AudioSegmentTests(unittest.TestCase):
                                    len(seg),
                                    percentage=0.01)
 
+    def test_export_as_wav_with_codec(self):
+        seg = self.seg1
+        exported_wav = seg.export(format='wav', codec='pcm_s32le')
+        seg_exported_wav = AudioSegment.from_wav(exported_wav)
+
+        self.assertWithinTolerance(len(seg_exported_wav),
+                                   len(seg),
+                                   percentage=0.01)
+        self.assertEqual(seg_exported_wav.sample_width, 4)
+
+    def test_export_as_wav_with_parameters(self):
+        seg = self.seg1
+        exported_wav = seg.export(format='wav', parameters=['-ar', '16000', '-ac', '1'])
+        seg_exported_wav = AudioSegment.from_wav(exported_wav)
+
+        self.assertWithinTolerance(len(seg_exported_wav),
+                                   len(seg),
+                                   percentage=0.01)
+        self.assertEqual(seg_exported_wav.frame_rate, 16000)
+        self.assertEqual(seg_exported_wav.channels, 1)
+
     def test_export_as_raw(self):
         seg = self.seg1
         exported_raw = seg.export(format='raw')
@@ -568,6 +630,16 @@ class AudioSegmentTests(unittest.TestCase):
                                    len(seg),
                                    percentage=0.01)
 
+    def test_export_as_raw_with_codec(self):
+        seg = self.seg1
+        with self.assertRaises(AttributeError):
+            seg.export(format='raw', codec='pcm_s32le')
+
+    def test_export_as_raw_with_parameters(self):
+        seg = self.seg1
+        with self.assertRaises(AttributeError):
+            seg.export(format='raw', parameters=['-ar', '16000', '-ac', '1'])
+
     def test_export_as_ogg(self):
         seg = self.seg1
         exported_ogg = seg.export(format='ogg')
@@ -665,11 +737,13 @@ class AudioSegmentTests(unittest.TestCase):
                 fd.close()
 
             tmp_seg = AudioSegment.from_mp3(tmp_mp3_file.name)
-            self.assertFalse(len(tmp_seg) < len(seg))
+            self.assertAlmostEqual(len(tmp_seg), len(seg), places=1)
 
             if sys.platform == 'win32':
                 os.remove(tmp_mp3_file.name)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_formats(self):
         seg_m4a = AudioSegment.from_file(
             os.path.join(data_dir, 'format_test.m4a'), "m4a")
@@ -688,6 +762,8 @@ class AudioSegmentTests(unittest.TestCase):
         wav = AudioSegment.from_wav(wav_file)
         self.assertEqual(wav.duration_seconds, self.seg1.duration_seconds)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_autodetect_format(self):
         aac_path = os.path.join(data_dir, 'wrong_extension.aac')
         fn = partial(AudioSegment.from_file, aac_path, "aac")
@@ -720,21 +796,29 @@ class AudioSegmentTests(unittest.TestCase):
             AudioSegment.from_file(self.mp3_file_path).export(tmp_webm_file,
                                                               format="webm")
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_ogg(self):
         with NamedTemporaryFile('w+b', suffix='.ogg') as tmp_ogg_file:
             AudioSegment.from_file(self.mp4_file_path).export(tmp_ogg_file,
                                                               format="ogg")
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3(self):
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
             AudioSegment.from_file(self.mp4_file_path).export(tmp_mp3_file,
                                                               format="mp3")
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_wav(self):
         with NamedTemporaryFile('w+b', suffix='.wav') as tmp_wav_file:
             AudioSegment.from_file(self.mp4_file_path).export(tmp_wav_file,
                                                               format="mp3")
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3_with_tags(self):
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
             tags_dict = {
@@ -746,6 +830,8 @@ class AudioSegmentTests(unittest.TestCase):
                                                               format="mp3",
                                                               tags=tags_dict)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3_with_tags_raises_exception_when_tags_are_not_a_dictionary(self):
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
             json = '{"title": "The Title You Want", "album": "Name of the Album", "artist": "Artist\'s name"}'
@@ -754,6 +840,8 @@ class AudioSegmentTests(unittest.TestCase):
                 format="mp3", tags=json)
             self.assertRaises(InvalidTag, func)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp4_as_mp3_with_tags_raises_exception_when_id3version_is_wrong(self):
         tags = {'artist': 'Artist', 'title': 'Title'}
         with NamedTemporaryFile('w+b', suffix='.mp3') as tmp_mp3_file:
@@ -766,6 +854,8 @@ class AudioSegmentTests(unittest.TestCase):
             )
             self.assertRaises(InvalidID3TagVersion, func)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_export_mp3_with_tags(self):
         tags = {'artist': 'Mozart', 'title': 'The Magic Flute'}
 
@@ -862,14 +952,14 @@ class AudioSegmentTests(unittest.TestCase):
         speedup_seg = self.seg1.speedup(2.0)
 
         self.assertWithinTolerance(
-            len(self.seg1) / 2, len(speedup_seg), percentage=0.01)
+            len(self.seg1) / 2, len(speedup_seg), percentage=0.02)
 
     def test_dBFS(self):
         seg_8bit = self.seg1.set_sample_width(1)
-        self.assertWithinTolerance(seg_8bit.dBFS, -8.88, tolerance=0.01)
-        self.assertWithinTolerance(self.seg1.dBFS, -8.88, tolerance=0.01)
-        self.assertWithinTolerance(self.seg2.dBFS, -10.39, tolerance=0.01)
-        self.assertWithinTolerance(self.seg3.dBFS, -6.47, tolerance=0.01)
+        self.assertWithinTolerance(seg_8bit.dBFS, -18.06, tolerance=1.5)
+        self.assertWithinTolerance(self.seg1.dBFS, -17.76, tolerance=1.5)
+        self.assertWithinTolerance(self.seg2.dBFS, -20.78, tolerance=1.5)
+        self.assertWithinTolerance(self.seg3.dBFS, -12.94, tolerance=1.5)
 
     def test_compress(self):
         compressed = self.seg1.compress_dynamic_range()
@@ -883,6 +973,8 @@ class AudioSegmentTests(unittest.TestCase):
         # average volume should be reduced
         self.assertTrue(compressed.rms < self.seg1.rms)
 
+    @unittest.skipUnless('aac' in get_supported_decoders(),
+                         "Unsupported codecs")
     def test_exporting_to_ogg_uses_default_codec_when_codec_param_is_none(self):
         delete = sys.platform != 'win32'
 
@@ -1015,6 +1107,20 @@ class SilenceTests(unittest.TestCase):
         self.seg1 = test1wav
         self.seg4 = test4wav
 
+    def test_split_on_silence_complete_silence(self):
+        seg = AudioSegment.silent(5000)
+        self.assertEquals( split_on_silence(seg), [] )
+
+    def test_split_on_silence_test1(self):
+        self.assertEqual(
+            len(split_on_silence(self.seg1, min_silence_len=500, silence_thresh=-20)),
+            3
+        )
+    def test_split_on_silence_no_silence(self):
+        splits = split_on_silence(self.seg1, min_silence_len=5000, silence_thresh=-200, keep_silence=True)
+        lens = [len(split) for split in splits]
+        self.assertEqual( lens, [len(self.seg1)] )
+
     def test_detect_completely_silent_segment(self):
         seg = AudioSegment.silent(5000)
         silent_ranges = detect_silence(seg, min_silence_len=1000, silence_thresh=-20)
@@ -1084,6 +1190,7 @@ class NoConverterTests(unittest.TestCase):
     def setUp(self):
         self.wave_file = os.path.join(data_dir, 'test1.wav')
         self.wave24_file = os.path.join(data_dir, 'test1-24bit.wav')
+        self.wave_empty = os.path.join(data_dir, 'test1_empty.wav')
         self.mp3_file = os.path.join(data_dir, 'test1.mp3')
         self.raw_file = os.path.join(data_dir, 'test1.raw')
         AudioSegment.converter = "definitely-not-a-path-to-anything-asdjklqwop"
@@ -1173,6 +1280,19 @@ class NoConverterTests(unittest.TestCase):
 
         self.assertEqual(len(exported), len(seg))
 
+    def test_opening_empty_wav_file(self):
+        seg = AudioSegment.from_wav(self.wave_empty)
+        self.assertTrue(len(seg) == 0)
+
+        seg = AudioSegment.from_file(self.wave_empty)
+        self.assertTrue(len(seg) == 0)
+
+        seg = AudioSegment.from_file(self.wave_empty, "wav")
+        self.assertTrue(len(seg) == 0)
+
+        seg = AudioSegment.from_file(self.wave_empty, format="wav")
+        self.assertTrue(len(seg) == 0)
+
 
 class FilterTests(unittest.TestCase):
 
